{
--------------------------------------------------------------------------------

Component Name: TmAppDlg
Author: Mats Asplund
Creation: Sep 02, 2003
Version: 1.0
Description:  TmAppDlg is a component for displaying application info to user.
              TmAppDlg is a redone from TAppDlg.
Credit:
EMail:        masprod@telia.com
Site:         http://go.to/masdp
Legal issues: All rights reserved 1999-2001 by Mats Asplund
              This software is provided 'as-is', without any express or
              implied warranty.  In no event will the author be held liable
              for any  damages araising from the use of this software.

              Permission is granted to anyone to use this software for any
              purpose, including commercial applications, and to alter it
              and redistribute it freely, subject to the following
              restrictions:

              1. The origin of this software must not be misrepresented,
                 you must not claim that you wrote the original software.
                 If you use this software in a product, an acknowledgment
                 in the product documentation would be appreciated but is
                 not required.

              2. Altered source versions must be plainly marked as such, and
                 must not be misrepresented as being the original software.

              3. This notice may not be removed or altered from any source
                 distribution.

              4. If you decide to use this software in any of your applications.
                 Send me an EMail address and tell me about it.

Quick Reference:
              TmAppDlg is a descendant from TComponent.

              Published properties:
              Productname: string
              Version: string
              Copyright: string
              Info: string
              EMail: string
              Site: string

              Method:
              Execute: Call this to run component.

--------------------------------------------------------------------------------
}
unit mAppDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  PTransBuffer = ^TTransBuffer;
  TTransBuffer = array[1..4] of smallint;

  TMailURLLabel = class(TLabel)
  private
    fMail: boolean;
  public
    procedure Click; override;
  published
    property Mail: boolean read fMail write fMail;
  end;

  TmAppDlg = class(TComponent)
  private
    fProductname: string;
    fVersion: string;
    fCopyright: string;
    fInfo: string;
    fEmail: string;
    fSite: string;
    EMailLabel, SiteLabel: TMailURLLabel;
    VersionList: TStringList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    function Execute: boolean;
    property Productname: string read fProductname write fProductname;
    property Version: string read fVersion write fVersion;
    property Copyright: string read fCopyright write fCopyright;
    property Email: string read fEmail write fEmail;
    property Site: string read fSite write fSite;
    property Info: string read fInfo write fInfo;
  end;

procedure Register;

implementation
uses AppDlgUnit, ShellApi;

const
  CInfoStr: array[1..4] of string =
  ('FileVersion', 'LegalCopyright', 'ProductName', 'ProductVersion');

  // Alter these to suit your needs
  defVersion = '1.0.1.1';
  defCopyright1 = 'All rights reserved ';
  defCopyright2 = ' by Mats Asplund';
  defEmail = 'masprod@telia.com';
  defSite = 'http://go.to/masdp';

function GetCreationDate(FileName: string): TDateTime;
var
  SearchRec: TSearchRec;
  DT: TFileTime;
  ST: TSystemTime;
begin
  Result:= 0;
  if not FileExists(FileName) then
    Exit;
  try
    SysUtils.FindFirst(FileName, faAnyFile, SearchRec);
    try
      FileTimeToLocalFileTime(SearchRec.FindData.ftCreationTime, DT);
      FileTimeToSystemTime(DT, ST);
      Result:= SystemTimeToDateTime(ST);
    finally
      SysUtils.FindClose(SearchRec);
    end;
  except
    Result:= 0;
  end;
end;

procedure GetVersionInfo(AVersionList: TStrings);
var
  FileName: string;
  i: integer;
  infoSize: DWORD;
  ptrans: PTransBuffer;
  transStr: string;
  typeStr: string;
  Value: PChar;
  verBuf: pointer;
  verSize: DWORD;
  wnd: DWORD;
begin
  AVersionList.Clear;
  FileName:= Application.ExeName;
  infoSize:= GetFileVersioninfoSize(PChar(FileName), wnd);

  if infoSize <> 0 then
  begin
    GetMem(verBuf, infoSize);
    try
      if GetFileVersionInfo(PChar(FileName), wnd, infoSize, verBuf) then
      begin
        VerQueryvalue(verBuf, PChar('\VarFileInfo\Translation'),
          pointer(ptrans), verSize);

        transStr:= IntToHex(ptrans^[1], 4) + IntToHex(ptrans^[2], 4);

        for i:= Low(CInfoStr) to High(CInfoStr) do
        begin
          typeStr:= 'StringFileInfo\' + transStr + '\' + CInfoStr[i];

          if VerQueryvalue(verBuf, PChar(typeStr),
            pointer(Value), verSize) then
            AVersionList.Add(Value);
        end
      end;
    finally
      FreeMem(verBuf);
    end;
  end;
end;

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TmAppDlg]);
end;

constructor TmAppDlg.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  VersionList:= TStringList.Create;
  ShortDateFormat:= 'yyyy-mm-dd';
end;

destructor TmAppDlg.Destroy;
begin
  VersionList.Free;
  inherited Destroy;
end;

function TmAppDlg.Execute: boolean;
var
  fYear, fYearCreate: string;
begin
  AppDlgForm:= TAppDlgForm.Create(Application);
  SiteLabel:= TMailURLLabel.Create(self);
  EMailLabel:= TMailURLLabel.Create(self);
  try
    with SiteLabel do
    begin
      Parent:= AppDlgForm;
      Visible:= true;
      Left:= 144;
      Top:= 40;
      Font.Color:= clBlue;
      Font.Style:= [fsUnderline];
      Cursor:= crHandPoint;
      Mail:= false;
    end;
    with EMailLabel do
    begin
      Parent:= AppDlgForm;
      Visible:= true;
      Left:= 144;
      Top:= 56;
      Font.Color:= clBlue;
      Font.Style:= [fsUnderline];
      Cursor:= crHandPoint;
      Mail:= true;
    end;
    if fProductname = '' then
      fProductname:= Application.Title;

    if fInfo = '' then
    begin
      fInfo:= Copy(DateTimeToStr(GetCreationDate(Application.ExeName)), 1, 10);
      fYearCreate:= Copy(fInfo, 1, 4);
      fYear:= Copy(DateTimeToStr(Now), 1, 4);
      if fYear <> fYearCreate then
        fYear:= fYearCreate + '-' + fYear;
    end;

    if fCopyright = '' then
    begin
      fCopyright:= defCopyright1 + fYear + defCopyright2;
    end;

    try
      if fVersion = '' then
      begin
        GetVersionInfo(VersionList);
        fVersion:= VersionList[0];
      end;
    except
      on E: Exception do
        MessageDlg('No versioninfo in exe-file !', mtWarning, [mbOK], 0);
    end;

    if fSite = '' then
      fSite:= defSite;

    if fEmail = '' then
      fEmail:= defEmail;

    AppDlgForm.Productname.Caption:= fProductname;
    AppDlgForm.Version.Caption:= fVersion;
    AppDlgForm.Copyright.Caption:= fCopyright;
    AppDlgForm.Info.Caption:= fInfo;
    SiteLabel.Caption:= fSite;
    EMailLabel.Caption:= fEmail;
    AppDlgForm.Caption:= 'About ' + fProductname;

    with AppDlgForm do
    begin
      ProgramIcon.Picture.Graphic:= Application.Icon;
      Result:= (ShowModal = IDOK);
    end;

  finally
    AppDlgForm.Free;
  end;
end;

procedure TMailURLLabel.Click;
begin
  inherited Click;
  if fMail then
    ShellExecute(0, 'open', PChar('mailto:' + Caption), '', '', SW_SHOWNORMAL)
  else
    ShellExecute(0, 'open', PChar(Caption), '', '', SW_SHOWNORMAL);
end;

end.

