{
----------------------------------------------------------
MAS-CompMaker was used to generate this code
MAS-CompMaker, copyright(c) 2000, Mats Asplund / MAs Prod.
----------------------------------------------------------

Component Name: TmGetNewVer
        Author: Mats Asplund
      Creation: 2002-07-23
       Version: 2.3
   Description: A component that alerts when a new programversion
                is available.
        Credit:
        E-mail: masprod@telia.com
          Site: http://go.to/masdp
  Legal issues: Copyright (C) 2002 by Mats Asplund

         Usage: This software is provided 'as-is', without any express or
                implied warranty.  In no event will the author be held liable
                for any  damages arising from the use of this software.

                Permission is granted to anyone to use this software for any
                purpose, including commercial applications, and to alter it
                and redistribute it freely, subject to the following
                restrictions:

                1. The origin of this software must not be misrepresented,
                   you must not claim that you wrote the original software.
                   If you use this software in a product, an acknowledgment
                   in the product documentation would be appreciated but is
                   not required.

                2. Altered source versions must be plainly marked as such, and
                   must not be misrepresented as being the original software.

                3. This notice may not be removed or altered from any source
                   distribution.

                4. If you decide to use this software in any of your applications.
                   Send me an EMail and tell me about it.

Quick Reference:
                TmGetNewVer inherits from TComponent.

                Key-Properties:
                  URLCurrVerFile: The URL to the file where the
                                  current version information is. This file
                                  should by of type IniFile, with two sections
                                  Currentversions and DownloadURL.
                          FileExample:
                            [Currentversions]
                            compmaker=3.7.0.0
                            winfiles=1.1.0.0
                            [DownloadURL]
                            compmaker=http://213.65.224.213/dl/app/compmaker.zip
                            winfiles=http://213.65.224.213/dl/app/winfiles.zip

                  SiteName:       Name of the site. Used in the download-dialog.

                  FailureDlg:     If true a dialog is shown to user informing
                                  that no version information could be retrieved.

                  DownloadDlg:    If true a download-dialog is shown to user.

                  TempFile:       A file for temporarily storage of the current
                                  version file.

                Key-Events:
                  OnGetURL:       Event occurs when file with current
                                  version information is downloaded.
                                  CurrentVersion and DownloadURL could be
                                  retrieved from this event.

                Key-Methods:
                  Execute:        Looks for new version. If so, prompts the user.
                                  Downloads the new version. Returns true if
                                  user decides to download new version.

--------------------------------------------------------------------------------
}
unit mGetNewVer;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, IniFiles, URLMon, ShellApi;

type
  PTransBuffer = ^TTransBuffer;
  TTransBuffer = array[1..4] of smallint;
  TOnGetURLEvent = procedure(Sender: TObject; CurrentVersion, DownloadURL:
    string) of object;

  TmGetNewVer = class;

  TVInfoDownload = class(TThread)
  private
    fOwner: TmGetNewVer;
    fURI: string;
    fTempFile: string;
    fFailure: Boolean;
    fFailureDlg: Boolean;
  protected
    constructor Create(Owner: TmGetNewVer; URI, TempFile: string; FailureDlg: Boolean);
    procedure Execute; override;
    procedure Ready;
  end;

  TmGetNewVer = class(TComponent)
  private
    fCopyright: string;
    fFailureDlg: Boolean;
    fURLCurrVerFile: string;
    fSiteName: string;
    fTempFile: TFileName;
    fOnGetURL: TOnGetURLEvent;
    fDownloadDlg: Boolean;
    procedure SetCop(Value: string);
    function IsNewVersion(CurrVersion, ThisVersion: string): Boolean;
    procedure FileFound;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
  published
    property URLCurrVerFile: string read fURLCurrVerFile write fURLCurrVerFile;
    property SiteName: string read fSiteName write fSiteName;
    property FailureDlg: Boolean read fFailureDlg write fFailureDlg;
    property DownloadDlg: Boolean read fDownloadDlg write fDownloadDlg;
    property TempFile: TFileName read fTempFile write fTempFile;
    property OnGetURL: TOnGetURLEvent read fOnGetURL write fOnGetURL;
    property Copyright: string read fCopyright write SetCop;
  end;

procedure Register;

implementation

const
  CInfoStr: array[1..4] of string =
  ('FileVersion', 'LegalCopyright', 'ProductName', 'ProductVersion');

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TmGetNewVer]);
end;

constructor TmGetNewVer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fURLCurrVerFile:= 'http://217.208.112.194/ver/curr_ver.txt';
  fSiteName:= 'MAS Delphi page';
  fFailureDlg:= true;
  fDownloadDlg:= true;
  fTempFile:= 'c:\curr_ver.tmp';
  fCopyright:= 'Version 2.3, 2002-2003 Mats Asplund, EMail: masprod@telia.com';
end;

destructor TmGetNewVer.Destroy;
begin
  inherited Destroy;
end;

procedure TmGetNewVer.Execute;
begin
  TVInfoDownload.Create(Self, fURLCurrVerFile, fTempFile, fFailureDlg);
end;

procedure TmGetNewVer.FileFound;
var
  VList: TStringList;
  VFile: TInifile;
  Version, CurrVer, DownloadURL: string;

  function GetVersionThisFile(AVersionList: TStrings): Boolean;
  var
    FileName: string;
    i: integer;
    infoSize: DWORD;
    ptrans: PTransBuffer;
    transStr: string;
    typeStr: string;
    Value: PChar;
    verBuf: pointer;
    verSize: DWORD;
    wnd: DWORD;
  begin
    Result:= false;
    AVersionList.Clear;
    FileName:= Application.ExeName;
    infoSize:= GetFileVersionInfoSize(PChar(FileName), wnd);

    if infoSize <> 0 then
    begin
      GetMem(verBuf, infoSize);
      try
        if GetFileVersionInfo(PChar(FileName), wnd, infoSize, verBuf) then
        begin
          VerQueryValue(verBuf, PChar('\VarFileInfo\Translation'),
            pointer(ptrans), verSize);

          transStr:= IntToHex(ptrans^[1], 4) + IntToHex(ptrans^[2], 4);

          for i:= Low(CInfoStr) to High(CInfoStr) do
          begin
            typeStr:= 'StringFileInfo\' + transStr + '\' + CInfoStr[i];

            if VerQueryValue(verBuf, PChar(typeStr),
              pointer(Value), verSize) then
              AVersionList.Add(Value);
          end;
          Result:= true;
        end;
      finally
        FreeMem(verBuf);
      end;
    end;
  end;

begin
  VList:= TStringList.Create;
  VFile:= nil;
  try
    if GetVersionThisFile(VList) then
    begin
      Version:= VList[0];
      VFile:= TInifile.Create(TempFile);
      CurrVer:= VFile.ReadString('CurrentVersions', Application.Title, '');
      DownloadURL:= VFile.ReadString('DownloadURL', Application.Title, '');
      if Assigned(fOnGetURL) then
        fOnGetURL(Self, CurrVer, DownloadURL);
      if IsNewVersion(CurrVer, Version) and fDownloadDlg then
        if MessageDlg('There''s a new version ' + CurrVer + ' of this program' +
          #13 + #10 + 'avaliable at: ' + SiteName + #13 + #10 +
          'Do you wan''t to download it now ?', mtConfirmation, [mbYes, mbNo], 0)
          = mrYes then
        begin
          ShellExecute(0, 'open', PChar(DownloadURL), '', '', SW_SHOWNORMAL);
        end
        else
          Exit;
    end;
  finally
    VList.Free;
    VFile.Free;
  end;
end;

function TmGetNewVer.IsNewVersion(CurrVersion, ThisVersion: string): Boolean;
var
  Params: TStringList;
  V1, V2: integer;
begin
  Params:= TStringList.Create;
  try
    result:= false;
    Params.Clear;
    if Pos('.', CurrVersion) = 0 then
    begin
      Params.Add(CurrVersion);
      result:= false;
      Exit;
    end;
    while Pos('.', CurrVersion) <> 0 do
    begin
      Params.Add(Copy(CurrVersion, 1, Pos('.', CurrVersion) - 1));
      CurrVersion:= Copy(CurrVersion, Pos('.', CurrVersion) + 1,
        Length(CurrVersion));
    end;
    Params.Add(CurrVersion);

    if Pos('.', ThisVersion) = 0 then
    begin
      Params.Add(ThisVersion);
      result:= false;
      Exit;
    end;
    while Pos('.', ThisVersion) <> 0 do
    begin
      Params.Add(Copy(ThisVersion, 1, Pos('.', ThisVersion) - 1));
      ThisVersion:= Copy(ThisVersion, Pos('.', ThisVersion) + 1,
        Length(ThisVersion));
    end;
    Params.Add(ThisVersion);

    V1:= (StrToInt(Params[0]) * 1000000) + (StrToInt(Params[1]) * 10000) +
      (StrToInt(Params[2]) * 100) + StrToInt(Params[3]);
    V2:= (StrToInt(Params[4]) * 1000000) + (StrToInt(Params[5]) * 10000) +
      (StrToInt(Params[6]) * 100) + StrToInt(Params[7]);
    if V1 > V2 then
      result:= true
    else
      result:= false;
  finally
    Params.Free;
  end;
end;

procedure TmGetNewVer.SetCop(Value: string);
begin
  Exit;
end;

{ TVInfoDownload }

constructor TVInfoDownload.Create(Owner: TmGetNewVer; URI, TempFile: string; FailureDlg: Boolean);
begin
  inherited Create(true);
  fOwner:= Owner;
  fURI:= URI;
  fTempFile:= TempFile;
  fFailureDlg:= FailureDlg;
  FreeOnTerminate:= true;
  resume;
end;

procedure TVInfoDownload.Execute;
begin
  if URLDownloadToFile(nil, PChar(fURI), PChar(fTempFile), 0, nil)
    <> 0 then
    fFailure:= true
  else
    fFailure:= false;
  Synchronize(Ready);
end;

procedure TVInfoDownload.Ready;
begin
  if fFailure and fFailureDlg then
    MessageDlg('Couldn''t download file version information ' +
      'from:' + #13 + #10 + fURI + ' !', mtWarning, [mbOK], 0)
  else
    fOwner.FileFound;
end;

end.

