{
---------------------------------------------------------------
Code generated with MAS Component Wizard,  2000-2002 Mats Asplund
---------------------------------------------------------------

Component Name: TmParser
        Author: Mats Asplund
 Creation Date: 2003-05-01
       Version: 1.2
   Description: A component for HTML parsing.
        E-mail: masprod@telia.com
       Website: http://go.to/masdp
  Legal Issues: All rigths reserved  2003 by Mats Asplund


Usage:
  This software is provided 'as-is', without any express or
  implied warranty.  In no event will the author be held liable
  for any  damages arising from the use of this software.

  Permission is granted to anyone to use this software for any
  purpose, including commercial applications, and to alter it
  and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented,
     you must not claim that you wrote the original software.
     If you use this software in a product, an acknowledgment
     in the product documentation would be appreciated but is
     not required.

  2. Altered source versions must be plainly marked as such, and
     must not be misrepresented as being the original software.

  3. This notice may not be removed or altered from any source
     distribution.

  4. If you decide to use this software in any of your applications.
     Send me an EMail and tell me about it.

Quick Reference:
  TmParser inherits from TComponent

  Key-Properties:
    URI:      Webfile that is to be loaded and parsed.
    TempFile: Temporary local file used for webfile.
    HTMLFile: Local file to be parsed.

  Key-Events:
    OnTag:   Event fired for each and every HTML-tag found.
             Note: OnTag takes all kind of tags. There could be OnLink or
                   OnImage events fired as well.
    OnText:  Event fired when a text string is found.
    OnLinkTag:  Event fired when a link tag is found.
    OnImageTag: Event fired when an image tag is found.

  Key-Methods:
    DownLoadAndParse: Downloads a webfile and starts parsing. Returns false if
                      file couldn't be downloaded.
    Parse:            Takes a local file and starts parsing. Returns false if
                      file doesn't exists.

---------------------------------------------------------------
}

unit mParser;

interface

uses
  Windows, Messages, SysUtils, Classes;

type
  TOnTagEvent =
    procedure(Sender: TObject; TagText: string; Index: integer) of object;
  TOnTextEvent =
    procedure(Sender: TObject; PureText: string; Index: integer) of object;
  TOnLinkTagEvent =
    procedure(Sender: TObject; LinkTagText: string; Index: integer) of object;
  TOnImageTagEvent =
    procedure(Sender: TObject; ImageTagText: string; Index: integer) of object;

  TmParser = class(TComponent)
  private
    fAbout: string;
    fOnImage: TOnImageTagEvent;
    fOnLink: TOnLinkTagEvent;
    fOnTag: TOnTagEvent;
    fOnText: TOnTextEvent;
    fHTMLFile: TFileName;
    fURI: string;
    fTempFile: TFileName;
    procedure SetAbout(Value: string);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function DownLoadAndParse: boolean;
    function Parse: boolean;
  published
    property URI: string read fURI write fURI;
    property HTMLFile: TFileName read fHTMLFile write fHTMLFile;
    property TempFile: TFileName read fTempFile write fTempFile;
    property OnTag: TOnTagEvent read fOnTag write fOnTag;
    property OnText: TOnTextEvent read fOnText write fOnText;
    property OnLinkTag: TOnLinkTagEvent read fOnLink write fOnLink;
    property OnImageTag: TOnImageTagEvent read fOnImage write fOnImage;
    property About: string read fAbout write SetAbout;
  end;

procedure Register;

implementation
uses URLMon, Dialogs;

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TmParser]);
end;

constructor TmParser.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fTempFile:= 'mparser.tmp';
  fAbout:= 'Version 1.2,  2003 - Mats Asplund';
end;

destructor TmParser.Destroy;
begin
  inherited Destroy;
end;

function TmParser.DownLoadAndParse: boolean;
var
  F: textfile;
begin
  if URLDownloadToFile(nil, pchar(fURI), pchar(fTempFile), 0, nil) <> 0 then
  begin
    AssignFile(F, fTempFile);
    Rewrite(F);
    CloseFile(F);
    Result:= false;
  end
  else
  begin
    fHTMLFile:= fTempFile;
    Parse;
    Result:= true;
  end;
end;

function TmParser.Parse;
var
  F: textfile;
  HTML, TheText, Row, TheTag: string;
  TagStart, TagLength, TagIndex, TextIndex, LinkIndex, ImageIndex, n: integer;
begin
  if FileExists(fHTMLFile) then
  begin
    TagIndex:= 0;
    TextIndex:= 0;
    LinkIndex:= 0;
    ImageIndex:= 0;
    TagStart:= 0;

    AssignFile(F, fHTMLFile);
    Reset(F);
    while not Eof(F) do
    begin
      Readln(F, Row);
      HTML:= HTML + Row + ' ';
    end;
    CloseFile(F);

    for n:= 1 to Length(HTML) do
    begin
      if Pos('<', HTML) > 0 then
        TagStart:= Pos('<', HTML) + 1;

      if Pos('>', HTML) > 0 then
      begin
        TagLength:= Pos('>', HTML) - TagStart;
        TheTag:= Copy(HTML, TagStart, TagLength);
        if Assigned(fOnTag) then
        begin
          fOnTag(self, TheTag, TagIndex);
          Inc(TagIndex);
        end;
        if Pos('src', Lowercase(TheTag)) > 0 then
          if Assigned(fOnImage) then
          begin
            fOnImage(self, TheTag, ImageIndex);
            Inc(ImageIndex);
          end;
        if Pos('href', Lowercase(TheTag)) > 0 then
          if Assigned(fOnLink) then
          begin
            fOnLink(self, TheTag, LinkIndex);
            Inc(LinkIndex);
          end;
        HTML:= Copy(HTML, TagStart + TagLength + 1, Length(HTML));

        if Length(HTML) > 0 then
          if HTML[1] <> '<' then
          begin
            TheText:= Copy(HTML, 1, Pos('<', HTML) - 1);
            if Assigned(fOnText) then
            begin
              fOnText(self, TheText, TextIndex);
              Inc(TextIndex);
            end;
          end;
      end;
    end;
    Result:= true;
  end
  else
    Result:= false;
end;

procedure TmParser.SetAbout(Value: string);
begin
  Exit;
end;

end.

