{
---------------------------------------------------------------
Code generated by AF Component Wizard
AF Component Wizard 2002 AFComponents - Ferruccio Accalai
Idea by MAS-CompMaker, 2000-2002 Mats Asplund http://go.to/masdp
---------------------------------------------------------------

Component Name: TmWebLogin
        Author: Mats Asplund
 Creation Date: 2003-03-25
       Version: 1.2
   Description: A component that generates a web login dialog, validates the
                login and sends a cookie to the user.
        E-mail: masprod@telia.com
       Website: http://go.to/masdp
  Legal Issues: All rigths reserved 2003 by Mats Asplund


Usage:
  This software is provided 'as-is', without any express or
  implied warranty.  In no event will the author be held liable
  for any  damages arising from the use of this software.

  Permission is granted to anyone to use this software for any
  purpose, including commercial applications, and to alter it
  and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented,
     you must not claim that you wrote the original software.
     If you use this software in a product, an acknowledgment
     in the product documentation would be appreciated but is
     not required.

  2. Altered source versions must be plainly marked as such, and
     must not be misrepresented as being the original software.

  3. This notice may not be removed or altered from any source
     distribution.

  4. If you decide to use this software in any of your applications.
     Send me an EMail and tell me about it.

Quick Reference:
  TmWebLogin inherits from TComponent.
  TmCookie inherits from TPersistent.

  Key-Properties:
    Cookie:             Class that controls the Cookie that will be sent over
                        to the user if the login is a success.
      Cookie.ValuePair: <Username> = <Password>
      Cookie.Values:    A TStringList containing the Valuepairs (just one needed).
      Cookie.ADomain:   Use ADomain to limit the recipients of the cookie to
                        certain domains.
      Cookie.APath:     Use Path to limit the recipients of the cookie to certain
                        paths.
      Cookie.AExpires:  How long the Cookie will "live". If you set this value
                        to -1, the Cookie will be "killed" when the user closes
                        his browser.
      Cookie.ASecure:   Set Secure to True if the Web client must use a secure
                        connection when sending the Cookie header.

    UserName:    The username.
    Password:    The password.
    Action:      The webaction used to validate the given username and password.
    Method:      Get or Post. Get will show the username and password in the
                 browsers addressfield. Use Post instead.
    Caption:     The login dialog caption.
    BgColor:     The login dialog background color.
    Width:       The login dialog width in pixels.
    UserCaption: Caption above the username field.
    PwdCaption:  Caption above the password field.
    BtnCaption:  SubmitButton caption.

  Key-Methods:
    Content:     The HTML-code for the login dialog.
    CheckCookie: Returns true if the cookie is present in the webrequest.
    ValidateLoginSetCookieIfOK:
                 Returns true and delivers the cookie, if the username and
                 password is OK.

---------------------------------------------------------------
}

unit mWebLogin;

interface

uses
  Windows, Messages, SysUtils, Classes, HTTPapp, HTTPProd;

type
  TMethod = (mwlPost, mwlGet);

  TmCookie = class(TPersistent)
  private
    fValues: TStrings;
    fASecure: Boolean;
    fADomain: string;
    fAPath: string;
    fAExpires: TDateTime;
    fValuePair: string;
    procedure SetValues(const Value: TStrings);
    procedure SetValuePair(const Value: string);
  protected
    constructor Create(ValuePair: string; ASecure: Boolean; ADomain, APath: string;
      AExpires: TDateTime);
  published
    property ValuePair: string read fValuePair write SetValuePair;
    property Values: TStrings read fValues write SetValues;
    property ADomain: string read fADomain write fADomain;
    property APath: string read fAPath write fAPath;
    property AExpires: TDateTime read fAExpires write fAExpires;
    property ASecure: Boolean read fASecure write fASecure;
  end;

  TmWebLogin = class(TComponent)
  private
    fPwdCaption: string;
    fBtnCaption: string;
    fAction: string;
    fMethod: TMethod;
    fUserCaption: string;
    fCaption: string;
    fAbout: string;
    fUserName: string;
    fPassword: string;
    fCookie: TmCookie;
    fBgColor: THTMLBgColor;
    fWidth: integer;
    procedure SetAbout(Value: string);
    procedure SetCookie(const Value: TmCookie);
    function CheckLogin(Request: TWebRequest): boolean;
  protected
  public
    function Content: string;
    function ValidateLoginSetCookieIfOK(Request: TWebRequest;
      Response: TWebResponse): boolean;
    function CheckCookie(Request: TWebRequest): boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property UserName: string read fUserName write fUserName;
    property Password: string read fPassword write fPassword;
    property Action: string read fAction write fAction;
    property Method: TMethod read fMethod write fMethod;
    property Cookie: TmCookie read fCookie write SetCookie;
    property Caption: string read fCaption write fCaption;
    property BgColor:THTMLBgColor read fBgColor write fBgColor;
    property Width: integer read fWidth write fWidth;
    property UserCaption: string read fUserCaption write fUserCaption;
    property PwdCaption: string read fPwdCaption write fPwdCaption;
    property BtnCaption: string read fBtnCaption write fBtnCaption;
    property About: string read fAbout write SetAbout;
  end;

procedure Register;

implementation

constructor TmWebLogin.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fCookie:= TmCookie.Create('Login=OK', false, '', '/cgi-bin/', -1);
  fCaption:= 'Login';
  fAction:= '/cgi-bin/login.cgi';
  fMethod:= mwlPost;
  fUserCaption:= 'Username:';
  fPwdCaption:= 'Password:';
  fBtnCaption:= '     OK     ';
  fBgColor:= 'Silver';
  fWidth:=250;
  fAbout:= 'Version 1.0, 2003  Mats Asplund';
end;

destructor TmWebLogin.Destroy;
begin
  inherited Destroy;
end;

function TmWebLogin.Content: string;
var
  Method: string;
begin
  if fMethod = mwlPost then
    Method:= 'Post'
  else
    Method:= 'Get';

  Result:= '<table width="' + IntToStr(fWidth) + '" bgcolor="' + fBgColor +
    '"><tr><td><B><FONT FACE="Arial"><FONT SIZE="+1" COLOR="#0000FF">' +
    fCaption + '<BR><BR><FONT FACE="Arial"><B><FONT SIZE="-1" COLOR="#000000">' +
    '<FORM NAME="F1" ACTION="' + fAction + '" METHOD="' +
    Method + '"><P>' + fUserCaption + '<br><INPUT TYPE="TEXT" NAME="USER"><br>' + 
    fPwdCaption + '<br><INPUT TYPE="PASSWORD" NAME="PW">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' +
    '<INPUT TYPE="SUBMIT" NAME="SubmitBtn" VALUE="' + fBtnCaption +
    '"></P></FORM></td></tr></table>';
end;

procedure TmWebLogin.SetAbout(Value: string);
begin
  Exit;
end;

procedure TmWebLogin.SetCookie(const Value: TmCookie);
begin
  fCookie.Assign(Value);
end;

function TmWebLogin.ValidateLoginSetCookieIfOK(Request: TWebRequest;
  Response: TWebResponse): boolean;
begin
  if CheckLogin(Request) then
  begin
    with fCookie do
      Response.SetCookieField(fValues, fADomain, fAPath, fAExpires, fASecure);
    Result:= true;
  end
  else
    Result:= false;
end;

function TmWebLogin.CheckLogin(Request: TWebRequest): boolean;
begin
  Result:= false;
  if fMethod = mwlPost then
  begin
    if (Request.ContentFields.Values['USER'] = fUserName) and
      (Request.ContentFields.Values['PW'] = fPassword) then
      Result:= true;
  end
  else
  begin
    if (Request.QueryFields.Values['USER'] = fUserName) and
      (Request.QueryFields.Values['PW'] = fPassword) then
      Result:= true;
  end;
end;

function TmWebLogin.CheckCookie(Request: TWebRequest): boolean;
var
  Value1, Value2: string;
begin
  Result:= false;
  Value1:= Copy(fCookie.fValuePair, 1, Pos('=', fCookie.fValuePair) -1);
  Value2:= Copy(fCookie.fValuePair, Pos('=', fCookie.fValuePair) +1, 1000);
  if Request.CookieFields.Values[Value1] = Value2 then
    Result:= true;
end;

{ TmCookie }

constructor TmCookie.Create(ValuePair: string; ASecure: Boolean; ADomain,
  APath: string; AExpires: TDateTime);
begin
  inherited Create;
  fValues:= TStringList.Create;
  SetValuePair(ValuePair);
  fASecure:= ASecure;
  fADomain:= ADomain;
  fAPath:= APath;
  fAExpires:= AExpires;
end;

procedure TmCookie.SetValuePair(const Value: string);
begin
  fValuePair := Value;
  fValues.Add(fValuePair);
end;

procedure TmCookie.SetValues(const Value: TStrings);
begin
  fValues.Assign(Value);
end;

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TmWebLogin]);
end;

end.

